# The contents of this file are licensed under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with the
# License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

base_interfaces = {
    "ATM": "ATM",
    "AT": "ATM",
    "B": "Bdi",
    "Bd": "Bdi",
    "Bdi": "Bdi",
    "EOBC": "EOBC",
    "EO": "EOBC",
    "Ethernet": "Ethernet",
    "Eth": "Ethernet",
    "eth": "Ethernet",
    "Et": "Ethernet",
    "et": "Ethernet",
    "FastEthernet": "FastEthernet",
    "FastEth": "FastEthernet",
    "FastE": "FastEthernet",
    "Fast": "FastEthernet",
    "Fas": "FastEthernet",
    "FE": "FastEthernet",
    "Fa": "FastEthernet",
    "fa": "FastEthernet",
    "Fddi": "Fddi",
    "FD": "Fddi",
    "FortyGigabitEthernet": "FortyGigabitEthernet",
    "FortyGigEthernet": "FortyGigabitEthernet",
    "FortyGigEth": "FortyGigabitEthernet",
    "FortyGigE": "FortyGigabitEthernet",
    "FortyGig": "FortyGigabitEthernet",
    "FGE": "FortyGigabitEthernet",
    "FO": "FortyGigabitEthernet",
    "Fo": "FortyGigabitEthernet",
    "FiftyGigabitEthernet": "FiftyGigabitEthernet",
    "FiftyGigEthernet": "FiftyGigabitEthernet",
    "FiftyGigEth": "FiftyGigabitEthernet",
    "FiftyGigE": "FiftyGigabitEthernet",
    "FI": "FiftyGigabitEthernet",
    "Fi": "FiftyGigabitEthernet",
    "fi": "FiftyGigabitEthernet",
    "GigabitEthernet": "GigabitEthernet",
    "GigEthernet": "GigabitEthernet",
    "GigEth": "GigabitEthernet",
    "GigE": "GigabitEthernet",
    "Gig": "GigabitEthernet",
    "GE": "GigabitEthernet",
    "Ge": "GigabitEthernet",
    "ge": "GigabitEthernet",
    "Gi": "GigabitEthernet",
    "gi": "GigabitEthernet",
    "HundredGigabitEthernet": "HundredGigabitEthernet",
    "HundredGigEthernet": "HundredGigabitEthernet",
    "HundredGigEth": "HundredGigabitEthernet",
    "HundredGigE": "HundredGigabitEthernet",
    "HundredGig": "HundredGigabitEthernet",
    "Hu": "HundredGigabitEthernet",
    "TwentyFiveGigabitEthernet": "TwentyFiveGigabitEthernet",
    "TwentyFiveGigEthernet": "TwentyFiveGigabitEthernet",
    "TwentyFiveGigEth": "TwentyFiveGigabitEthernet",
    "TwentyFiveGigE": "TwentyFiveGigabitEthernet",
    "TwentyFiveGig": "TwentyFiveGigabitEthernet",
    "TF": "TwentyFiveGigabitEthernet",
    "Tf": "TwentyFiveGigabitEthernet",
    "tf": "TwentyFiveGigabitEthernet",
    "TwoHundredGigabitEthernet": "TwoHundredGigabitEthernet",
    "TwoHundredGigEthernet": "TwoHundredGigabitEthernet",
    "TwoHundredGigEth": "TwoHundredGigabitEthernet",
    "TwoHundredGigE": "TwoHundredGigabitEthernet",
    "TwoHundredGig": "TwoHundredGigabitEthernet",
    "TH": "TwoHundredGigabitEthernet",
    "Th": "TwoHundredGigabitEthernet",
    "th": "TwoHundredGigabitEthernet",
    "FourHundredGigabitEthernet": "FourHundredGigabitEthernet",
    "FourHundredGigEthernet": "FourHundredGigabitEthernet",
    "FourHundredGigEth": "FourHundredGigabitEthernet",
    "FourHundredGigE": "FourHundredGigabitEthernet",
    "FourHundredGig": "FourHundredGigabitEthernet",
    "F": "FourHundredGigabitEthernet",
    "f": "FourHundredGigabitEthernet",
    "Loopback": "Loopback",
    "loopback": "Loopback",
    "Lo": "Loopback",
    "lo": "Loopback",
    "Management": "Management",
    "Mgmt": "Management",
    "mgmt": "Management",
    "Ma": "Management",
    "Management_short": "Ma",
    "MFR": "MFR",
    "Multilink": "Multilink",
    "Mu": "Multilink",
    "n": "nve",
    "nv": "nve",
    "nve": "nve",
    "PortChannel": "Port-channel",
    "Port-channel": "Port-channel",
    "Port-Channel": "Port-channel",
    "port-channel": "Port-channel",
    "po": "Port-channel",
    "Po": "Port-channel",
    "POS": "POS",
    "PO": "POS",
    "Serial": "Serial",
    "Se": "Serial",
    "S": "Serial",
    "TenGigabitEthernet": "TenGigabitEthernet",
    "TenGigEthernet": "TenGigabitEthernet",
    "TenGigEth": "TenGigabitEthernet",
    "TenGig": "TenGigabitEthernet",
    "TeGig": "TenGigabitEthernet",
    "Ten": "TenGigabitEthernet",
    "T": "TenGigabitEthernet",
    "Te": "TenGigabitEthernet",
    "te": "TenGigabitEthernet",
    "Tunnel": "Tunnel",
    "Tun": "Tunnel",
    "Tu": "Tunnel",
    "Twe": "TwentyFiveGigE",
    "Tw": "TwoGigabitEthernet",
    "Two": "TwoGigabitEthernet",
    "Virtual-Access": "Virtual-Access",
    "Vi": "Virtual-Access",
    "Virtual-Template": "Virtual-Template",
    "Vt": "Virtual-Template",
    "VLAN": "VLAN",
    "V": "VLAN",
    "Vl": "VLAN",
    "Wlan-GigabitEthernet": "Wlan-GigabitEthernet",
}

reverse_mapping = {
    "ATM": "At",
    "EOBC": "EO",
    "Ethernet": "Et",
    "FastEthernet": "Fa",
    "Fddi": "FD",
    "FortyGigabitEthernet": "Fo",
    "GigabitEthernet": "Gi",
    "HundredGigabitEthernet": "Hu",
    "Loopback": "Lo",
    "Management": "Ma",
    "MFR": "MFR",
    "Multilink": "Mu",
    "Port-channel": "Po",
    "POS": "PO",
    "Serial": "Se",
    "TenGigabitEthernet": "Te",
    "Tunnel": "Tu",
    "TwoGigabitEthernet": "Two",
    "TwentyFiveGigE": "Twe",
    "Virtual-Access": "Vi",
    "Virtual-Template": "Vt",
    "VLAN": "Vl",
    "Wlan-GigabitEthernet": "Wl-Gi",
}
