#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_wireless_rf_profiles_assignments_by_device_info
short_description: Information module for organizations _wireless _rf _profiles _assignments
  _by _device
description:
  - Get all organizations _wireless _rf _profiles _assignments _by _device.
  - List the RF profiles of an organization by device.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  perPage:
    description:
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 3 - 1000. Default is 1000.
    type: int
  startingAfter:
    description:
      - >
        StartingAfter query parameter. A token used by the server to indicate the
        start of the page. Often this is a timestamp or an ID but it is not limited
        to those. This parameter should not be defined by client applications. The
        link for the first, last, prev, or next page in the HTTP Link header should
        define it.
    type: str
  endingBefore:
    description:
      - >
        EndingBefore query parameter. A token used by the server to indicate the end
        of the page. Often this is a timestamp or an ID but it is not limited to those.
        This parameter should not be defined by client applications. The link for
        the first, last, prev, or next page in the HTTP Link header should define
        it.
    type: str
  networkIds:
    description:
      - NetworkIds query parameter. Optional parameter to filter devices by network.
    elements: str
    type: list
  productTypes:
    description:
      - >
        ProductTypes query parameter. Optional parameter to filter devices by product
        type. Valid types are wireless, appliance, switch, systemsManager, camera,
        cellularGateway, sensor, wirelessController, campusGateway, and secureConnect.
    elements: str
    type: list
  name:
    description:
      - >
        Name query parameter. Optional parameter to filter RF profiles by device name.
        All returned devices will have a name that contains the search term or is
        an exact match.
    type: str
  mac:
    description:
      - >
        Mac query parameter. Optional parameter to filter RF profiles by device MAC
        address. All returned devices will have a MAC address that contains the search
        term or is an exact match.
    type: str
  serial:
    description:
      - >
        Serial query parameter. Optional parameter to filter RF profiles by device
        serial number. All returned devices will have a serial number that contains
        the search term or is an exact match.
    type: str
  model:
    description:
      - >
        Model query parameter. Optional parameter to filter RF profiles by device
        model. All returned devices will have a model that contains the search term
        or is an exact match.
    type: str
  macs:
    description:
      - >
        Macs query parameter. Optional parameter to filter RF profiles by one or more
        device MAC addresses. All returned devices will have a MAC address that is
        an exact match.
    elements: str
    type: list
  serials:
    description:
      - >
        Serials query parameter. Optional parameter to filter RF profiles by one or
        more device serial numbers. All returned devices will have a serial number
        that is an exact match.
    elements: str
    type: list
  models:
    description:
      - >
        Models query parameter. Optional parameter to filter RF profiles by one or
        more device models. All returned devices will have a model that is an exact
        match.
    elements: str
    type: list
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for wireless getOrganizationWirelessRfProfilesAssignmentsByDevice
    description: Complete reference of the getOrganizationWirelessRfProfilesAssignmentsByDevice
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-wireless-rf-profiles-assignments-by-device
notes:
  - SDK Method used are
    wireless.Wireless.get_organization_wireless_rf_profiles_assignments_by_device,
  - Paths used are
    get /organizations/{organizationId}/wireless/rfProfiles/assignments/byDevice,
"""

EXAMPLES = r"""
- name: Get all organizations _wireless _rf _profiles _assignments _by _device
  cisco.meraki.organizations_wireless_rf_profiles_assignments_by_device_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    perPage: 0
    startingAfter: string
    endingBefore: string
    networkIds: []
    productTypes: []
    name: string
    mac: string
    serial: string
    model: string
    macs: []
    serials: []
    models: []
    organizationId: string
    total_pages: -1
    direction: next
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "items": [
          {
            "network": {
              "id": "string"
            },
            "name": "string",
            "serial": "string",
            "model": "string",
            "rfProfile": {
              "id": "string",
              "name": "string",
              "isIndoorDefault": true,
              "isOutdoorDefault": true
            }
          }
        ],
        "meta": {
          "counts": {
            "items": {
              "total": 0,
              "remaining": 0
            }
          }
        }
      }
    ]
"""
